import csv
import os
import re


def get_subjects_files(data_dir):
    for (root, subdirs, files) in os.walk(data_dir):
        return files

def get_rows(subjects_file, data_dir):
    file_path = os.path.join(data_dir, subjects_file)
    with open(file_path, 'rt') as fh:
        csvreader = csv.reader(fh, delimiter=";")
        headers = next(csvreader)
        rows = [row for row in csvreader]
    return headers, rows

def get_ref_index(row, headers, debug):
    if "references" in headers:
        return headers.index("references")
    elif "verwijzing" in headers:
        return headers.index("verwijzing")

def normalise_row(row, headers, debug):
    ref_index = get_ref_index(row, headers, debug)
    if debug:
        print("ref_row:", row)
    row[ref_index] = normalise_reference(row[ref_index], debug)

def normalise_page_ref(page_ref):
    try:
        if "-" in page_ref:
            page_ref = re.sub("[^0-9-]","", page_ref)
            page_first, page_last = page_ref.split("-")
            if int(page_first) > 2000:
                print("high first page number:", page_ref)
                return page_ref
            if int(page_last) > 2000:
                print("high last page number:", page_ref)
                return page_ref
            return [page_num for page_num in range(int(page_first), int(page_last)+1)]
        else:
            return [int(page_ref)]
    except ValueError:
        return []
    return page_ref

def normalise_reference(reference, debug):
    if len(reference) == 0 or "zie" in reference:
        return reference
    try:
        ref_string = reference.strip().lower().replace(".",",") # replace periods with commas
        ref_string = re.sub(r"<.*?>", "", ) # remove HTML markup
        ref_string = re.sub(r"(\d) (\d)", r"\1, \2", ref_string) # add missings commas
        ref_string = ref_string.replace(" ","") # remove whitespace
        ref_string = re.sub(r",+", ",", ref_string) # collapse multiple commas
        ref_string = re.sub(r"-+", "-", ref_string) # collapse multiple hyphens
        ref_string = re.sub(r",$", "", ref_string) # remove trailing comma
        ref_string = re.sub(r"^,", "", ref_string) # remove leading comma
        page_refs = ref_string.split(",")
        page_refs = [int(page_num) for page_ref in page_refs for page_num in normalise_page_ref(page_ref)]
        reference = ", ".join([str(page_ref) for page_ref in page_refs])
        #print("normalised:", reference)
    except ValueError:
        print("reference:", "#" + reference + "#")
        print("ref_string:", ref_string)
    return reference

data_dir = "bewerkte_bestanden"

for subjects_file in get_subjects_files(data_dir):
    out_file = subjects_file.replace(".csv", "_normalised.csv")
    headers, rows = get_rows(subjects_file, data_dir)
    print(subjects_file, len(rows))
    with open(out_file, 'wt') as fh_out:
        csvwriter = csv.writer(fh_out, delimiter="\t")
        csvwriter.writerow(headers)
        for row in rows:
            debug=False
            if subjects_file == "232_geog.csv":
                debug = True
            normalise_row(row, headers, debug)
            csvwriter.writerow(row)
        #normalised_rows = [normalise_row(row, headers) for row in rows]
